library(base)
library(car)
library(caret)
library(Cubist)
library(data.table)
library(entropy)
library(ggfortify)
library(ggplot2)
library(gstat)
library(MASS)
library(moments)
library(normtest)
library(pls)
library(pracma)
library(prospectr)
library(signal)
library(tripack)
library(zoo)
require(clhs)
library(stats)
library(plyr)

#get command from HPC #
args <- commandArgs(trailingOnly = T)

met <- as.numeric(args[1]) # 4methods(1-4): Random, KS, cLHS, KM
prop <- as.numeric(args[2]) #5properties(1-5): clay, sand, pH, OC, CEC
xyz<-as.numeric(args[3]) #12sizes(1-12):50,100,150,200,250,300,400,500,1000,1500,2000,3000

#load the data
#load(file='../RDS/Function.RData')
LUCAS.SOIL<-load(file="../LUCAS/LUCAS.SOIL_corr.Rdata") # already resampled down to 1/3: 5639 total samples

#check skewness of the Y values; if >2; data has to be transformed
aa<-c(6,8,9,11,16);
par(mfrow=c(2,3))
for (i in 1:length(aa)){
  print(skewness(LUCAS.SOIL[aa][i]))
}

#collect the Y values
YY<-cbind(LUCAS.SOIL[,c(6,8,9,11,16)])

#create function for spectra pre-processing
NIR_treat<-function(spec){
  temp1<-spec[,201:4101]
  nw=ncol(temp1)
  step2<-seq(1,nw,by=4)
  temp2.1<-temp1[,step2]
  temp3<-filter_spectra(temp2.1,type='S-Golay',n=11,p=2,m=0)
  temp4<-filter_spectra(temp3,type='SNV'); return (temp4)
}

#split the dataset into calibration and validation
set.seed(34783)
icc<-sample(1:nrow(LUCAS.SOIL),4639) #keep 1000 for validation

X<-NIR_treat(LUCAS.SOIL$spc[icc,]);Y<-YY[icc,]
xv<-NIR_treat(LUCAS.SOIL$spc[-icc,]);yval<-YY[-icc,]


#create variable to save the result in
pls_res<-NULL;cub_res<-NULL
metode=c('Random','KS','cLHS','KM')
s_size=c(50,100,150,200,250,300,400,500,1000,1500,2000,3000)

#calculate principal component for the pre-treated spectra
PCA<-prcomp(X)

#determine at which component the PC>99%
scores<-as.data.frame(PCA$x)
scores2<-as.data.frame(prcomp(xv)$x)
for (i in 1:100){
  if(sum((PCA$sdev[1:i])^2/sum((PCA$sdev)^2))>0.99){break;}
}
pca_comp=i


ukur=s_size[xyz]
for (repe in 1:50){
  if(met==1){
    #Method1: Random sampling
    par(mfrow=c(2,3));set.seed(repe*206)
    ic<-sample(1:nrow(X),ukur)
    xc<-X[ic,];yc<-Y[ic,prop];yv<-yval[,prop]
    plot(scores$PC1,scores$PC2,xlab = 'PC1',ylab = 'PC2',main=metode[met])
    points(scores$PC1[ic],scores$PC2[ic],col='red',cex=2)
    points(scores2$PC1,scores2$PC2,col='blue',pch="*")
  } else if(met==2){
    #Method2: Kennard-Stone
    par(mfrow=c(2,3))
    library(prospectr)
    ic<-kenStone(X=scores[,1:pca_comp], k = ukur, metric = "euclid")$model
    xc<-X[ic,];yc<-Y[ic,prop];yv<-yval[,prop]
    plot(scores$PC1,scores$PC2,xlab = 'PC1',ylab = 'PC2',main=metode[met])
    points(scores$PC1[ic],scores$PC2[ic],col='red',cex=2)
    points(scores2$PC1,scores2$PC2,col='blue',pch="*")
  }else if(met==3){
    #Method3: conditioned Latin Hypercube Sampling
    par(mfrow=c(2,3))
    require(clhs);set.seed(repe*206)
    ic=clhs(scores[,1:pca_comp],size = ukur,iter = 100000,progress=F)
    xc<-X[ic,];yc<-Y[ic,prop];yv<-yval[,prop]
    plot(scores$PC1,scores$PC2,xlab = 'PC1',ylab = 'PC2',main=metode[met])
    points(scores$PC1[ic],scores$PC2[ic],col='red',cex=2)
    points(scores2$PC1,scores2$PC2,col='blue',pch="*")
  }else if(met==4){
    #Method4: K-means Clustering
    par(mfrow=c(2,3))
    abcd<-matrix(1:nrow(X))
    abc<-kmeans(scores[,1:pca_comp],ukur)
    m<-cbind(matrix(c(1:nrow(X))),abc$cluster)
    
    ic<-NULL
    for (jj in 1:ukur){
      m1<-which(m[,2]==jj);set.seed(repe*206)
      ic<-c(ic,sample(m1,1))}
    xc<-X[ic,];yc<-Y[ic,prop];yv<-yval[,prop]
    plot(scores$PC1,scores$PC2,xlab = 'PC1',ylab = 'PC2',main=metode[met])
    points(scores$PC1[ic],scores$PC2[ic],col='red',cex=2)
    points(scores2$PC1,scores2$PC2,col='blue',pch="*")
  }
  
  #create plsr model based on the selected calibration sample
  library(pls)
  pls.model<-plsr(yc~.,data=xc,ncomp=30,validation='CV')
  plot(RMSEP(pls.model), legendpos = "topright") # RMSEP curves
  #find at which latent variable, RMSEP is the lowest; set max latent variable=30
  nc<-match(min(as.numeric(as.data.frame(unlist(RMSEP(pls.model)$val))[1,])),as.numeric(as.data.frame(unlist(RMSEP(pls.model)$val))[1,])) # no. components to use
  
  maxc=30
  if(nc>maxc){nc=maxc}
  pls.c.pred<-predict(pls.model, ncomp = nc,newdata=xc);
  pls.v.pred<-predict(pls.model, ncomp = nc,newdata=xv);
  pls_res<-rbind(pls_res,data.frame(method=metode[met],ukur,repe,goof(yc,pls.c.pred,coefficient = c('R2','RMSE','bias','RPIQ'),main='PLS Cal'),
                                    v=goof(yv,pls.v.pred,coefficient = c('R2','RMSE','bias','RPIQ'),main='PLS Val'),soil.prop=colnames(yval[prop])))
  
  #create Cubist model based on the selected calibration sample
  library(Cubist)
  cubist.model<-cubist(x=xc,y=yc)
  cub.c.pred<-predict(cubist.model,xc)
  cub.v.pred<-predict(cubist.model,xv)
  cub_res<-rbind(cub_res,data.frame(method=metode[met],ukur,repe,goof(yc,cub.c.pred,coefficient = c('R2','RMSE','bias','RPIQ'),main='Cub Cal'),
                                    v=goof(yv,cub.v.pred,coefficient = c('R2','RMSE','bias','RPIQ'),main='Cub Val'),soil.prop=colnames(yval[prop])))
  if(met==2){break;}# if method == KS; just do one repeat
}

#save the replicates results
save(pls_res,cub_res,file=paste0("Lucas_",metode[met],"_",colnames(Y)[prop],"_",ukur,".Rdata"))